""" - Type Stubs

Auto-generated stub file for type checking and IDE support.

Endpoint: firewall/interface_policy6
Category: cmdb
"""

from __future__ import annotations

from typing import (
    Any,
    ClassVar,
    Literal,
    TypedDict,
    overload,
)

from hfortix_fortios.models import (
    FortiObject,
    FortiObjectList,
)


# ================================================================
# TypedDict Payloads
# ================================================================

class InterfacePolicy6Srcaddr6Item:
    """Nested item for srcaddr6 field - supports attribute access."""
    name: str


class InterfacePolicy6Dstaddr6Item:
    """Nested item for dstaddr6 field - supports attribute access."""
    name: str


class InterfacePolicy6Service6Item:
    """Nested item for service6 field - supports attribute access."""
    name: str


class InterfacePolicy6Payload(TypedDict, total=False):
    """Payload type for InterfacePolicy6 operations."""
    policyid: int
    uuid: str
    status: Literal["enable", "disable"]
    comments: str
    logtraffic: Literal["all", "utm", "disable"]
    interface: str
    srcaddr6: str | list[str] | list[dict[str, Any]] | list[InterfacePolicy6Srcaddr6Item]
    dstaddr6: str | list[str] | list[dict[str, Any]] | list[InterfacePolicy6Dstaddr6Item]
    service6: str | list[str] | list[dict[str, Any]] | list[InterfacePolicy6Service6Item]
    application_list_status: Literal["enable", "disable"]
    application_list: str
    ips_sensor_status: Literal["enable", "disable"]
    ips_sensor: str
    dsri: Literal["enable", "disable"]
    av_profile_status: Literal["enable", "disable"]
    av_profile: str
    webfilter_profile_status: Literal["enable", "disable"]
    webfilter_profile: str
    casb_profile_status: Literal["enable", "disable"]
    casb_profile: str
    emailfilter_profile_status: Literal["enable", "disable"]
    emailfilter_profile: str
    dlp_profile_status: Literal["enable", "disable"]
    dlp_profile: str


# ================================================================
# Response Types (TypedDict for dict-style access)
# ================================================================

class InterfacePolicy6Response(TypedDict, total=False):
    """Response type for InterfacePolicy6 - use with .dict property for typed dict access."""
    policyid: int
    uuid: str
    status: Literal["enable", "disable"]
    comments: str
    logtraffic: Literal["all", "utm", "disable"]
    interface: str
    srcaddr6: list[InterfacePolicy6Srcaddr6Item]
    dstaddr6: list[InterfacePolicy6Dstaddr6Item]
    service6: list[InterfacePolicy6Service6Item]
    application_list_status: Literal["enable", "disable"]
    application_list: str
    ips_sensor_status: Literal["enable", "disable"]
    ips_sensor: str
    dsri: Literal["enable", "disable"]
    av_profile_status: Literal["enable", "disable"]
    av_profile: str
    webfilter_profile_status: Literal["enable", "disable"]
    webfilter_profile: str
    casb_profile_status: Literal["enable", "disable"]
    casb_profile: str
    emailfilter_profile_status: Literal["enable", "disable"]
    emailfilter_profile: str
    dlp_profile_status: Literal["enable", "disable"]
    dlp_profile: str


# ================================================================
# Response Types (Class for attribute access)
# ================================================================


class InterfacePolicy6Object(FortiObject):
    """Typed FortiObject for InterfacePolicy6 with field access."""
    policyid: int
    uuid: str
    status: Literal["enable", "disable"]
    comments: str
    logtraffic: Literal["all", "utm", "disable"]
    interface: str
    srcaddr6: list[InterfacePolicy6Srcaddr6Item]
    dstaddr6: list[InterfacePolicy6Dstaddr6Item]
    service6: list[InterfacePolicy6Service6Item]
    application_list_status: Literal["enable", "disable"]
    application_list: str
    ips_sensor_status: Literal["enable", "disable"]
    ips_sensor: str
    dsri: Literal["enable", "disable"]
    av_profile_status: Literal["enable", "disable"]
    av_profile: str
    webfilter_profile_status: Literal["enable", "disable"]
    webfilter_profile: str
    casb_profile_status: Literal["enable", "disable"]
    casb_profile: str
    emailfilter_profile_status: Literal["enable", "disable"]
    emailfilter_profile: str
    dlp_profile_status: Literal["enable", "disable"]
    dlp_profile: str


# ================================================================
# Main Endpoint Class
# ================================================================

class InterfacePolicy6:
    """
    
    Endpoint: firewall/interface_policy6
    Category: cmdb
    MKey: policyid
    """
    
    # Class attributes for introspection
    endpoint: ClassVar[str] = ...
    path: ClassVar[str] = ...
    category: ClassVar[str] = ...
    mkey: ClassVar[str] = ...
    capabilities: ClassVar[dict[str, Any]] = ...
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
    
    # ================================================================
    # GET Methods
    # ================================================================
    
    # CMDB with mkey - overloads for single vs list returns
    @overload
    def get(
        self,
        policyid: int,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> InterfacePolicy6Object: ...
    
    @overload
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObjectList[InterfacePolicy6Object]: ...
    
    def get_schema(
        self,
        vdom: str | None = ...,
        format: str = ...,
    ) -> FortiObject: ...

    # ================================================================
    # POST Method
    # ================================================================
    
    def post(
        self,
        payload_dict: InterfacePolicy6Payload | None = ...,
        policyid: int | None = ...,
        uuid: str | None = ...,
        status: Literal["enable", "disable"] | None = ...,
        comments: str | None = ...,
        logtraffic: Literal["all", "utm", "disable"] | None = ...,
        interface: str | None = ...,
        srcaddr6: str | list[str] | list[dict[str, Any]] | list[InterfacePolicy6Srcaddr6Item] | None = ...,
        dstaddr6: str | list[str] | list[dict[str, Any]] | list[InterfacePolicy6Dstaddr6Item] | None = ...,
        service6: str | list[str] | list[dict[str, Any]] | list[InterfacePolicy6Service6Item] | None = ...,
        application_list_status: Literal["enable", "disable"] | None = ...,
        application_list: str | None = ...,
        ips_sensor_status: Literal["enable", "disable"] | None = ...,
        ips_sensor: str | None = ...,
        dsri: Literal["enable", "disable"] | None = ...,
        av_profile_status: Literal["enable", "disable"] | None = ...,
        av_profile: str | None = ...,
        webfilter_profile_status: Literal["enable", "disable"] | None = ...,
        webfilter_profile: str | None = ...,
        casb_profile_status: Literal["enable", "disable"] | None = ...,
        casb_profile: str | None = ...,
        emailfilter_profile_status: Literal["enable", "disable"] | None = ...,
        emailfilter_profile: str | None = ...,
        dlp_profile_status: Literal["enable", "disable"] | None = ...,
        dlp_profile: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> InterfacePolicy6Object: ...

    # ================================================================
    # PUT Method
    # ================================================================
    
    def put(
        self,
        payload_dict: InterfacePolicy6Payload | None = ...,
        policyid: int | None = ...,
        uuid: str | None = ...,
        status: Literal["enable", "disable"] | None = ...,
        comments: str | None = ...,
        logtraffic: Literal["all", "utm", "disable"] | None = ...,
        interface: str | None = ...,
        srcaddr6: str | list[str] | list[dict[str, Any]] | list[InterfacePolicy6Srcaddr6Item] | None = ...,
        dstaddr6: str | list[str] | list[dict[str, Any]] | list[InterfacePolicy6Dstaddr6Item] | None = ...,
        service6: str | list[str] | list[dict[str, Any]] | list[InterfacePolicy6Service6Item] | None = ...,
        application_list_status: Literal["enable", "disable"] | None = ...,
        application_list: str | None = ...,
        ips_sensor_status: Literal["enable", "disable"] | None = ...,
        ips_sensor: str | None = ...,
        dsri: Literal["enable", "disable"] | None = ...,
        av_profile_status: Literal["enable", "disable"] | None = ...,
        av_profile: str | None = ...,
        webfilter_profile_status: Literal["enable", "disable"] | None = ...,
        webfilter_profile: str | None = ...,
        casb_profile_status: Literal["enable", "disable"] | None = ...,
        casb_profile: str | None = ...,
        emailfilter_profile_status: Literal["enable", "disable"] | None = ...,
        emailfilter_profile: str | None = ...,
        dlp_profile_status: Literal["enable", "disable"] | None = ...,
        dlp_profile: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> InterfacePolicy6Object: ...

    # ================================================================
    # DELETE Method
    # ================================================================
    
    def delete(
        self,
        policyid: int | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject: ...

    # ================================================================
    # Utility Methods
    # ================================================================
    
    def exists(
        self,
        policyid: int,
        vdom: str | bool | None = ...,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: InterfacePolicy6Payload | None = ...,
        policyid: int | None = ...,
        uuid: str | None = ...,
        status: Literal["enable", "disable"] | None = ...,
        comments: str | None = ...,
        logtraffic: Literal["all", "utm", "disable"] | None = ...,
        interface: str | None = ...,
        srcaddr6: str | list[str] | list[dict[str, Any]] | list[InterfacePolicy6Srcaddr6Item] | None = ...,
        dstaddr6: str | list[str] | list[dict[str, Any]] | list[InterfacePolicy6Dstaddr6Item] | None = ...,
        service6: str | list[str] | list[dict[str, Any]] | list[InterfacePolicy6Service6Item] | None = ...,
        application_list_status: Literal["enable", "disable"] | None = ...,
        application_list: str | None = ...,
        ips_sensor_status: Literal["enable", "disable"] | None = ...,
        ips_sensor: str | None = ...,
        dsri: Literal["enable", "disable"] | None = ...,
        av_profile_status: Literal["enable", "disable"] | None = ...,
        av_profile: str | None = ...,
        webfilter_profile_status: Literal["enable", "disable"] | None = ...,
        webfilter_profile: str | None = ...,
        casb_profile_status: Literal["enable", "disable"] | None = ...,
        casb_profile: str | None = ...,
        emailfilter_profile_status: Literal["enable", "disable"] | None = ...,
        emailfilter_profile: str | None = ...,
        dlp_profile_status: Literal["enable", "disable"] | None = ...,
        dlp_profile: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> list[str] | list[dict[str, Any]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject: ...
    
    @staticmethod
    def schema() -> FortiObject: ...


__all__ = [
    "InterfacePolicy6",
    "InterfacePolicy6Payload",
    "InterfacePolicy6Response",
    "InterfacePolicy6Object",
]


"""FortiOS CMDB - Ike category"""

from .clear import Clear

__all__ = [
    "Clear",
    "Ike",
]


class Ike:
    """Ike endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """Ike endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.clear = Clear(client)



"""FortiOS CMDB - TacacsPlusaccounting2 category"""

from .filter import Filter
from .setting import Setting

__all__ = [
    "Filter",
    "Setting",
    "TacacsPlusaccounting2",
]


class TacacsPlusaccounting2:
    """TacacsPlusaccounting2 endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """TacacsPlusaccounting2 endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.filter = Filter(client)
        self.setting = Setting(client)



"""FortiOS CMDB - HaPeer category (stub)"""

from typing import Any
from ..ha_peer_base import HaPeer as HaPeerBase
from .disconnect import Disconnect
from .update import Update

class HaPeer(HaPeerBase):
    """HaPeer endpoints wrapper for CMDB API."""

    disconnect: Disconnect
    update: Update

    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...



"""Type stubs for hfortix_fortios.api.utils module."""

from __future__ import annotations

from typing import Any, Optional, TYPE_CHECKING

if TYPE_CHECKING:
    from hfortix_core.http.client import HTTPClient
    from hfortix_core.http.interface import IHTTPClient
    from hfortix_fortios.client import FortiOS

__all__ = ["Utils"]

# Type alias for all acceptable client types
FortiOSClient = FortiOS | HTTPClient | IHTTPClient


class PerformanceTestResults:
    """Results from performance testing."""
    
    def print_summary(self) -> None: ...
    def to_dict(self) -> dict[str, Any]: ...


class Utils:
    """
    Utility functions for FortiOS API operations.

    Provides tools for:
    - Performance testing and benchmarking
    - Connection pool validation
    - Device profiling and recommendations

    Example:
        >>> fgt = FortiOS(host="192.168.1.99", token="token")
        >>> results = fgt.api.utils.performance_test()
        >>> results.print_summary()
    """

    def __init__(
        self,
        client: FortiOS | HTTPClient | IHTTPClient,
    ) -> None:
        """Initialize utilities with HTTP client or FortiOS client reference."""
        ...

    def performance_test(
        self,
        test_validation: bool = True,
        test_endpoints: bool = True,
        test_concurrency: bool = False,
        sequential_count: int = 10,
        concurrent_count: int = 50,
        concurrent_level: int = 20,
        endpoints: Optional[list[str]] = None,
    ) -> PerformanceTestResults:
        """
        Run performance tests on this FortiGate device.

        Args:
            test_validation: Test connection pool validation (default: True)
            test_endpoints: Test various API endpoints (default: True)
            test_concurrency: Test concurrent performance (default: False)
            sequential_count: Number of sequential requests per endpoint
            concurrent_count: Number of concurrent requests
            concurrent_level: Concurrency level for async test
            endpoints: Custom list of endpoints to test

        Returns:
            PerformanceTestResults object with detailed metrics and recommendations
        """
        ...

    def validate_connection_pool(self) -> dict[str, Any]:
        """Validate connection pool settings and return stats."""
        ...

    def get_device_profile(self) -> dict[str, Any]:
        """Get device profile and recommendations."""
        ...

    def benchmark_endpoint(
        self,
        endpoint: str,
        count: int = 10,
    ) -> dict[str, Any]:
        """Benchmark a specific API endpoint."""
        ...



"""Type stubs for SYSTEM category."""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from hfortix_core.http.interface import IHTTPClient
    from .fabric_admin_lockout_exists_on_firmware_update import FabricAdminLockoutExistsOnFirmwareUpdate
    from .fabric_time_in_sync import FabricTimeInSync
    from .psirt_vulnerabilities import PsirtVulnerabilities

__all__ = [
    "FabricAdminLockoutExistsOnFirmwareUpdate",
    "FabricTimeInSync",
    "PsirtVulnerabilities",
    "System",
]


class System:
    """SYSTEM API category.
    
    All endpoints return FortiObject instances with:
    - Attribute access: response.field
    - Dictionary access: response["field"]
    - Convert to dict: response.dict or response.json
    """
    
    fabric_admin_lockout_exists_on_firmware_update: FabricAdminLockoutExistsOnFirmwareUpdate
    fabric_time_in_sync: FabricTimeInSync
    psirt_vulnerabilities: PsirtVulnerabilities

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize system category with HTTP client."""
        ...



""" - Type Stubs

Auto-generated stub file for type checking and IDE support.

Endpoint: wireless_controller/log
Category: cmdb
"""

from __future__ import annotations

from typing import (
    Any,
    ClassVar,
    Literal,
    TypedDict,
)

from hfortix_fortios.models import (
    FortiObject,
    FortiObjectList,
)


# ================================================================
# TypedDict Payloads
# ================================================================

class LogPayload(TypedDict, total=False):
    """Payload type for Log operations."""
    status: Literal["enable", "disable"]
    addrgrp_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"]
    ble_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"]
    clb_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"]
    dhcp_starv_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"]
    led_sched_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"]
    radio_event_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"]
    rogue_event_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"]
    sta_event_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"]
    sta_locate_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"]
    wids_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"]
    wtp_event_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"]
    wtp_fips_event_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"]


# ================================================================
# Response Types (TypedDict for dict-style access)
# ================================================================

class LogResponse(TypedDict, total=False):
    """Response type for Log - use with .dict property for typed dict access."""
    status: Literal["enable", "disable"]
    addrgrp_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"]
    ble_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"]
    clb_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"]
    dhcp_starv_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"]
    led_sched_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"]
    radio_event_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"]
    rogue_event_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"]
    sta_event_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"]
    sta_locate_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"]
    wids_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"]
    wtp_event_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"]
    wtp_fips_event_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"]


# ================================================================
# Response Types (Class for attribute access)
# ================================================================


class LogObject(FortiObject):
    """Typed FortiObject for Log with field access."""
    status: Literal["enable", "disable"]
    addrgrp_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"]
    ble_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"]
    clb_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"]
    dhcp_starv_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"]
    led_sched_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"]
    radio_event_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"]
    rogue_event_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"]
    sta_event_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"]
    sta_locate_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"]
    wids_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"]
    wtp_event_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"]
    wtp_fips_event_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"]


# ================================================================
# Main Endpoint Class
# ================================================================

class Log:
    """
    
    Endpoint: wireless_controller/log
    Category: cmdb
    """
    
    # Class attributes for introspection
    endpoint: ClassVar[str] = ...
    path: ClassVar[str] = ...
    category: ClassVar[str] = ...
    capabilities: ClassVar[dict[str, Any]] = ...
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
    
    # ================================================================
    # GET Methods
    # ================================================================
    
    # Singleton endpoint (no mkey)
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> LogObject: ...
    
    def get_schema(
        self,
        vdom: str | None = ...,
        format: str = ...,
    ) -> FortiObject: ...


    # ================================================================
    # PUT Method
    # ================================================================
    
    def put(
        self,
        payload_dict: LogPayload | None = ...,
        status: Literal["enable", "disable"] | None = ...,
        addrgrp_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"] | None = ...,
        ble_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"] | None = ...,
        clb_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"] | None = ...,
        dhcp_starv_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"] | None = ...,
        led_sched_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"] | None = ...,
        radio_event_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"] | None = ...,
        rogue_event_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"] | None = ...,
        sta_event_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"] | None = ...,
        sta_locate_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"] | None = ...,
        wids_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"] | None = ...,
        wtp_event_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"] | None = ...,
        wtp_fips_event_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> LogObject: ...


    # ================================================================
    # Utility Methods
    # ================================================================
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: LogPayload | None = ...,
        status: Literal["enable", "disable"] | None = ...,
        addrgrp_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"] | None = ...,
        ble_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"] | None = ...,
        clb_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"] | None = ...,
        dhcp_starv_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"] | None = ...,
        led_sched_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"] | None = ...,
        radio_event_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"] | None = ...,
        rogue_event_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"] | None = ...,
        sta_event_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"] | None = ...,
        sta_locate_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"] | None = ...,
        wids_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"] | None = ...,
        wtp_event_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"] | None = ...,
        wtp_fips_event_log: Literal["emergency", "alert", "critical", "error", "warning", "notification", "information", "debug"] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> list[str] | list[dict[str, Any]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject: ...
    
    @staticmethod
    def schema() -> FortiObject: ...


__all__ = [
    "Log",
    "LogPayload",
    "LogResponse",
    "LogObject",
]


"""FortiOS CMDB - Proxy category (stub)"""

from typing import Any
from ..proxy_base import Proxy as ProxyBase
from .count import Count

class Proxy(ProxyBase):
    """Proxy endpoints wrapper for CMDB API."""

    count: Count

    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...



"""
Pydantic Models for CMDB - system/sdn_connector

Runtime validation models for system/sdn_connector configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Literal, Optional
from enum import Enum

# ============================================================================
# Enum Definitions for Child Table Fields (for fields with 4+ allowed values)
# ============================================================================

# ============================================================================
# Child Table Models (sorted deepest-first so nested models are defined before their parents)
# ============================================================================

class SdnConnectorServerList(BaseModel):
    """
    Child table model for server-list.
    
    Server address list of the remote SDN connector.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    ip: str = Field(max_length=15, description="IPv4 address.")
class SdnConnectorRouteTableRoute(BaseModel):
    """
    Child table model for route-table.route.
    
    Configure Azure route.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str = Field(max_length=63, description="Route name.")    
    next_hop: str = Field(max_length=127, description="Next hop address.")
class SdnConnectorRouteTable(BaseModel):
    """
    Child table model for route-table.
    
    Configure Azure route table.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str = Field(max_length=63, description="Route table name.")    
    subscription_id: str | None = Field(max_length=63, default=None, description="Subscription ID of Azure route table.")    
    resource_group: str | None = Field(max_length=63, default=None, description="Resource group of Azure route table.")    
    route: list[SdnConnectorRouteTableRoute] = Field(default_factory=list, description="Configure Azure route.")
class SdnConnectorRoute(BaseModel):
    """
    Child table model for route.
    
    Configure GCP route.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str = Field(max_length=63, description="Route name.")
class SdnConnectorOciRegionList(BaseModel):
    """
    Child table model for oci-region-list.
    
    Configure OCI region list.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    region: str = Field(max_length=31, description="OCI region.")
class SdnConnectorNicIp(BaseModel):
    """
    Child table model for nic.ip.
    
    Configure IP configuration.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str = Field(max_length=63, description="IP configuration name.")    
    private_ip: str | None = Field(max_length=39, default=None, description="Private IP address.")    
    public_ip: str | None = Field(max_length=63, default=None, description="Public IP name.")    
    resource_group: str | None = Field(max_length=63, default=None, description="Resource group of Azure public IP.")
class SdnConnectorNic(BaseModel):
    """
    Child table model for nic.
    
    Configure Azure network interface.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str = Field(max_length=63, description="Network interface name.")    
    peer_nic: str | None = Field(max_length=63, default=None, description="Peer network interface name.")    
    ip: list[SdnConnectorNicIp] = Field(default_factory=list, description="Configure IP configuration.")
class SdnConnectorGcpProjectListGcpZoneList(BaseModel):
    """
    Child table model for gcp-project-list.gcp-zone-list.
    
    Configure GCP zone list.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str = Field(max_length=127, description="GCP zone name.")
class SdnConnectorGcpProjectList(BaseModel):
    """
    Child table model for gcp-project-list.
    
    Configure GCP project list.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: str = Field(max_length=127, serialization_alias="id", description="GCP project ID.")    
    gcp_zone_list: list[SdnConnectorGcpProjectListGcpZoneList] = Field(default_factory=list, description="Configure GCP zone list.")
class SdnConnectorForwardingRule(BaseModel):
    """
    Child table model for forwarding-rule.
    
    Configure GCP forwarding rule.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    rule_name: str = Field(max_length=63, description="Forwarding rule name.")    
    target: str = Field(max_length=63, description="Target instance name.")
class SdnConnectorExternalIp(BaseModel):
    """
    Child table model for external-ip.
    
    Configure GCP external IP.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str = Field(max_length=63, description="External IP name.")
class SdnConnectorExternalAccountListRegionList(BaseModel):
    """
    Child table model for external-account-list.region-list.
    
    AWS region name list.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    region: str = Field(max_length=31, description="AWS region name.")
class SdnConnectorExternalAccountList(BaseModel):
    """
    Child table model for external-account-list.
    
    Configure AWS external account list.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    role_arn: str = Field(max_length=2047, description="AWS role ARN to assume.")    
    external_id: str | None = Field(max_length=1399, default=None, description="AWS external ID.")    
    region_list: list[SdnConnectorExternalAccountListRegionList] = Field(description="AWS region name list.")
class SdnConnectorCompartmentList(BaseModel):
    """
    Child table model for compartment-list.
    
    Configure OCI compartment list.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    compartment_id: str = Field(max_length=127, description="OCI compartment ID.")
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================

class SdnConnectorTypeEnum(str, Enum):
    """Allowed values for type_ field."""
    ACI = "aci"
    ALICLOUD = "alicloud"
    AWS = "aws"
    AZURE = "azure"
    GCP = "gcp"
    NSX = "nsx"
    NUAGE = "nuage"
    OCI = "oci"
    OPENSTACK = "openstack"
    KUBERNETES = "kubernetes"
    VMWARE = "vmware"
    SEPM = "sepm"
    ACI_DIRECT = "aci-direct"
    IBM = "ibm"
    NUTANIX = "nutanix"
    SAP = "sap"

class SdnConnectorAzureRegionEnum(str, Enum):
    """Allowed values for azure_region field."""
    GLOBAL = "global"
    CHINA = "china"
    GERMANY = "germany"
    USGOV = "usgov"
    LOCAL = "local"

class SdnConnectorIbmRegionEnum(str, Enum):
    """Allowed values for ibm_region field."""
    DALLAS = "dallas"
    WASHINGTON_DC = "washington-dc"
    LONDON = "london"
    FRANKFURT = "frankfurt"
    SYDNEY = "sydney"
    TOKYO = "tokyo"
    OSAKA = "osaka"
    TORONTO = "toronto"
    SAO_PAULO = "sao-paulo"
    MADRID = "madrid"


# ============================================================================
# Main Model
# ============================================================================

class SdnConnectorModel(BaseModel):
    """
    Pydantic model for system/sdn_connector configuration.
    
    Configure connection to SDN Connector.
    
    Validation Rules:        - name: max_length=35 pattern=        - status: pattern=        - type_: pattern=        - proxy: max_length=35 pattern=        - use_metadata_iam: pattern=        - microsoft_365: pattern=        - ha_status: pattern=        - verify_certificate: pattern=        - vdom: max_length=31 pattern=        - server: max_length=127 pattern=        - server_list: pattern=        - server_port: min=0 max=65535 pattern=        - message_server_port: min=0 max=65535 pattern=        - username: max_length=64 pattern=        - password: pattern=        - vcenter_server: max_length=127 pattern=        - vcenter_username: max_length=64 pattern=        - vcenter_password: pattern=        - access_key: max_length=31 pattern=        - secret_key: max_length=59 pattern=        - region: max_length=31 pattern=        - vpc_id: max_length=31 pattern=        - alt_resource_ip: pattern=        - external_account_list: pattern=        - tenant_id: max_length=127 pattern=        - client_id: max_length=63 pattern=        - client_secret: max_length=59 pattern=        - subscription_id: max_length=63 pattern=        - resource_group: max_length=63 pattern=        - login_endpoint: max_length=127 pattern=        - resource_url: max_length=127 pattern=        - azure_region: pattern=        - nic: pattern=        - route_table: pattern=        - user_id: max_length=127 pattern=        - compartment_list: pattern=        - oci_region_list: pattern=        - oci_region_type: pattern=        - oci_cert: max_length=63 pattern=        - oci_fingerprint: max_length=63 pattern=        - external_ip: pattern=        - route: pattern=        - gcp_project_list: pattern=        - forwarding_rule: pattern=        - service_account: max_length=127 pattern=        - private_key: pattern=        - secret_token: pattern=        - domain: max_length=127 pattern=        - group_name: max_length=127 pattern=        - server_cert: max_length=127 pattern=        - server_ca_cert: max_length=127 pattern=        - api_key: max_length=59 pattern=        - ibm_region: pattern=        - par_id: max_length=63 pattern=        - update_interval: min=0 max=3600 pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    name: str | None = Field(max_length=35, default=None, description="SDN connector name.")    
    status: Literal["disable", "enable"] = Field(default="enable", description="Enable/disable connection to the remote SDN connector.")    
    type_: SdnConnectorTypeEnum = Field(default=SdnConnectorTypeEnum.AWS, serialization_alias="type", description="Type of SDN connector.")    
    proxy: str | None = Field(max_length=35, default=None, description="SDN proxy.")  # datasource: ['system.sdn-proxy.name']    
    use_metadata_iam: Literal["disable", "enable"] = Field(default="disable", description="Enable/disable use of IAM role from metadata to call API.")    
    microsoft_365: Literal["disable", "enable"] = Field(default="disable", description="Enable to use as Microsoft 365 connector.")    
    ha_status: Literal["disable", "enable"] = Field(default="disable", description="Enable/disable use for FortiGate HA service.")    
    verify_certificate: Literal["disable", "enable"] = Field(default="enable", description="Enable/disable server certificate verification.")    
    vdom: str | None = Field(max_length=31, default=None, description="Virtual domain name of the remote SDN connector.")  # datasource: ['system.vdom.name']    
    server: str = Field(max_length=127, description="Server address of the remote SDN connector.")    
    server_list: list[SdnConnectorServerList] = Field(description="Server address list of the remote SDN connector.")    
    server_port: int | None = Field(ge=0, le=65535, default=0, description="Port number of the remote SDN connector.")    
    message_server_port: int | None = Field(ge=0, le=65535, default=0, description="HTTP port number of the SAP message server.")    
    username: str = Field(max_length=64, description="Username of the remote SDN connector as login credentials.")    
    password: Any = Field(description="Password of the remote SDN connector as login credentials.")    
    vcenter_server: str | None = Field(max_length=127, default=None, description="vCenter server address for NSX quarantine.")    
    vcenter_username: str | None = Field(max_length=64, default=None, description="vCenter server username for NSX quarantine.")    
    vcenter_password: Any = Field(default=None, description="vCenter server password for NSX quarantine.")    
    access_key: str = Field(max_length=31, description="AWS / ACS access key ID.")    
    secret_key: Any = Field(max_length=59, description="AWS / ACS secret access key.")    
    region: str = Field(max_length=31, description="AWS / ACS region name.")    
    vpc_id: str | None = Field(max_length=31, default=None, description="AWS VPC ID.")    
    alt_resource_ip: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable AWS alternative resource IP.")    
    external_account_list: list[SdnConnectorExternalAccountList] = Field(default_factory=list, description="Configure AWS external account list.")    
    tenant_id: str | None = Field(max_length=127, default=None, description="Tenant ID (directory ID).")    
    client_id: str | None = Field(max_length=63, default=None, description="Azure client ID (application ID).")    
    client_secret: Any = Field(max_length=59, default=None, description="Azure client secret (application key).")    
    subscription_id: str | None = Field(max_length=63, default=None, description="Azure subscription ID.")    
    resource_group: str | None = Field(max_length=63, default=None, description="Azure resource group.")    
    login_endpoint: str | None = Field(max_length=127, default=None, description="Azure Stack login endpoint.")    
    resource_url: str | None = Field(max_length=127, default=None, description="Azure Stack resource URL.")    
    azure_region: SdnConnectorAzureRegionEnum | None = Field(default=SdnConnectorAzureRegionEnum.GLOBAL, description="Azure server region.")    
    nic: list[SdnConnectorNic] = Field(default_factory=list, description="Configure Azure network interface.")    
    route_table: list[SdnConnectorRouteTable] = Field(default_factory=list, description="Configure Azure route table.")    
    user_id: str | None = Field(max_length=127, default=None, description="User ID.")    
    compartment_list: list[SdnConnectorCompartmentList] = Field(default_factory=list, description="Configure OCI compartment list.")    
    oci_region_list: list[SdnConnectorOciRegionList] = Field(default_factory=list, description="Configure OCI region list.")    
    oci_region_type: Literal["commercial", "government"] = Field(default="commercial", description="OCI region type.")    
    oci_cert: str | None = Field(max_length=63, default=None, description="OCI certificate.")  # datasource: ['certificate.local.name']    
    oci_fingerprint: str | None = Field(max_length=63, default=None, description="OCI pubkey fingerprint.")    
    external_ip: list[SdnConnectorExternalIp] = Field(default_factory=list, description="Configure GCP external IP.")    
    route: list[SdnConnectorRoute] = Field(default_factory=list, description="Configure GCP route.")    
    gcp_project_list: list[SdnConnectorGcpProjectList] = Field(default_factory=list, description="Configure GCP project list.")    
    forwarding_rule: list[SdnConnectorForwardingRule] = Field(default_factory=list, description="Configure GCP forwarding rule.")    
    service_account: str = Field(max_length=127, description="GCP service account email.")    
    private_key: str = Field(description="Private key of GCP service account.")    
    secret_token: str = Field(description="Secret token of Kubernetes service account.")    
    domain: str | None = Field(max_length=127, default=None, description="Domain name.")    
    group_name: str | None = Field(max_length=127, default=None, description="Full path group name of computers.")    
    server_cert: str | None = Field(max_length=127, default=None, description="Trust servers that contain this certificate only.")  # datasource: ['certificate.remote.name']    
    server_ca_cert: str | None = Field(max_length=127, default=None, description="Trust only those servers whose certificate is directly/indirectly signed by this certificate.")  # datasource: ['certificate.remote.name', 'certificate.ca.name']    
    api_key: Any = Field(max_length=59, description="IBM cloud API key or service ID API key.")    
    ibm_region: SdnConnectorIbmRegionEnum = Field(default=SdnConnectorIbmRegionEnum.DALLAS, description="IBM cloud region name.")    
    par_id: str | None = Field(max_length=63, default=None, description="Public address range ID.")    
    update_interval: int | None = Field(ge=0, le=3600, default=60, description="Dynamic object update interval (30 - 3600 sec, default = 60, 0 = disabled).")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    @field_validator('proxy')
    @classmethod
    def validate_proxy(cls, v: Any) -> Any:
        """
        Validate proxy field.
        
        Datasource: ['system.sdn-proxy.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('vdom')
    @classmethod
    def validate_vdom(cls, v: Any) -> Any:
        """
        Validate vdom field.
        
        Datasource: ['system.vdom.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('oci_cert')
    @classmethod
    def validate_oci_cert(cls, v: Any) -> Any:
        """
        Validate oci_cert field.
        
        Datasource: ['certificate.local.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('server_cert')
    @classmethod
    def validate_server_cert(cls, v: Any) -> Any:
        """
        Validate server_cert field.
        
        Datasource: ['certificate.remote.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('server_ca_cert')
    @classmethod
    def validate_server_ca_cert(cls, v: Any) -> Any:
        """
        Validate server_ca_cert field.
        
        Datasource: ['certificate.remote.name', 'certificate.ca.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "SdnConnectorModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)
    # ========================================================================
    # Datasource Validation Methods
    # ========================================================================    
    async def validate_proxy_references(self, client: Any) -> list[str]:
        """
        Validate proxy references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/sdn-proxy        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = SdnConnectorModel(
            ...     proxy="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_proxy_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.system.sdn_connector.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "proxy", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.system.sdn_proxy.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Proxy '{value}' not found in "
                "system/sdn-proxy"
            )        
        return errors    
    async def validate_vdom_references(self, client: Any) -> list[str]:
        """
        Validate vdom references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/vdom        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = SdnConnectorModel(
            ...     vdom="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_vdom_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.system.sdn_connector.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "vdom", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.system.vdom.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Vdom '{value}' not found in "
                "system/vdom"
            )        
        return errors    
    async def validate_oci_cert_references(self, client: Any) -> list[str]:
        """
        Validate oci_cert references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - certificate/local        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = SdnConnectorModel(
            ...     oci_cert="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_oci_cert_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.system.sdn_connector.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "oci_cert", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.certificate.local.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Oci-Cert '{value}' not found in "
                "certificate/local"
            )        
        return errors    
    async def validate_server_cert_references(self, client: Any) -> list[str]:
        """
        Validate server_cert references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - certificate/remote        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = SdnConnectorModel(
            ...     server_cert="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_server_cert_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.system.sdn_connector.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "server_cert", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.certificate.remote.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Server-Cert '{value}' not found in "
                "certificate/remote"
            )        
        return errors    
    async def validate_server_ca_cert_references(self, client: Any) -> list[str]:
        """
        Validate server_ca_cert references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - certificate/remote        - certificate/ca        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = SdnConnectorModel(
            ...     server_ca_cert="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_server_ca_cert_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.system.sdn_connector.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "server_ca_cert", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.certificate.remote.exists(value):
            found = True
        elif await client.api.cmdb.certificate.ca.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Server-Ca-Cert '{value}' not found in "
                "certificate/remote or certificate/ca"
            )        
        return errors    
    async def validate_all_references(self, client: Any) -> list[str]:
        """
        Validate ALL datasource references in this model.
        
        Convenience method that runs all validate_*_references() methods
        and aggregates the results.
        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of all validation errors found
            
        Example:
            >>> errors = await policy.validate_all_references(fgt._client)
            >>> if errors:
            ...     for error in errors:
            ...         print(f"  - {error}")
        """
        all_errors = []
        
        errors = await self.validate_proxy_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_vdom_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_oci_cert_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_server_cert_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_server_ca_cert_references(client)
        all_errors.extend(errors)        
        return all_errors

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "SdnConnectorModel",    "SdnConnectorServerList",    "SdnConnectorExternalAccountList",    "SdnConnectorExternalAccountList.RegionList",    "SdnConnectorNic",    "SdnConnectorNic.Ip",    "SdnConnectorRouteTable",    "SdnConnectorRouteTable.Route",    "SdnConnectorCompartmentList",    "SdnConnectorOciRegionList",    "SdnConnectorExternalIp",    "SdnConnectorRoute",    "SdnConnectorGcpProjectList",    "SdnConnectorGcpProjectList.GcpZoneList",    "SdnConnectorForwardingRule",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:11.289604Z
# ============================================================================


"""
Pydantic Models for MONITOR - fortiview/realtime_statistics

Runtime validation models for fortiview/realtime_statistics configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field
from typing import Any, Optional

# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


# ============================================================================
# Main Model
# ============================================================================

class RealtimeStatisticsModel(BaseModel):
    """
    Pydantic model for fortiview/realtime_statistics configuration.
    
    Retrieve realtime drill-down and summary data for FortiView.
    
    Validation Rules:        - srcaddr: pattern=        - dstaddr: pattern=        - srcaddr6: pattern=        - dstaddr6: pattern=        - srcport: pattern=        - dstport: pattern=        - srcintf: pattern=        - srcintfrole: pattern=        - dstintf: pattern=        - dstintfrole: pattern=        - policyid: pattern=        - security_policyid: pattern=        - protocol: pattern=        - web_category: pattern=        - web_domain: pattern=        - application: pattern=        - country: pattern=        - seconds: pattern=        - since: pattern=        - owner: pattern=        - username: pattern=        - shaper: pattern=        - srcuuid: pattern=        - dstuuid: pattern=        - sessionid: pattern=        - report_by: pattern=        - sort_by: pattern=        - ip_version: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    srcaddr: Any = Field(default=None)    
    dstaddr: Any = Field(default=None)    
    srcaddr6: Any = Field(default=None)    
    dstaddr6: Any = Field(default=None)    
    srcport: Any = Field(default=None)    
    dstport: Any = Field(default=None)    
    srcintf: Any = Field(default=None)    
    srcintfrole: Any = Field(default=None)    
    dstintf: Any = Field(default=None)    
    dstintfrole: Any = Field(default=None)    
    policyid: Any = Field(default=None)    
    security_policyid: Any = Field(default=None)    
    protocol: Any = Field(default=None)    
    web_category: Any = Field(default=None)    
    web_domain: Any = Field(default=None)    
    application: Any = Field(default=None)    
    country: Any = Field(default=None)    
    seconds: Any = Field(default=None)    
    since: Any = Field(default=None)    
    owner: Any = Field(default=None)    
    username: Any = Field(default=None)    
    shaper: Any = Field(default=None)    
    srcuuid: Any = Field(default=None)    
    dstuuid: Any = Field(default=None)    
    sessionid: Any = Field(default=None)    
    report_by: str | None = Field(default=None)    
    sort_by: str | None = Field(default=None)    
    ip_version: str | None = Field(default=None)    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "RealtimeStatisticsModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "RealtimeStatisticsModel",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:14.323243Z
# ============================================================================


"""FortiOS CMDB - Sniffer category"""

from .delete import Delete
from .download import Download
from .list import List
from .meta import Meta
from .start import Start
from .stop import Stop

__all__ = [
    "Delete",
    "Download",
    "List",
    "Meta",
    "Sniffer",
    "Start",
    "Stop",
]


class Sniffer:
    """Sniffer endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """Sniffer endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.delete = Delete(client)
        self.download = Download(client)
        self.list = List(client)
        self.meta = Meta(client)
        self.start = Start(client)
        self.stop = Stop(client)



"""
Pydantic Models for CMDB - vpn/ipsec/phase2_interface

Runtime validation models for vpn/ipsec/phase2_interface configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Literal, Optional
from enum import Enum

# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================

class Phase2InterfaceProposalEnum(str, Enum):
    """Allowed values for proposal field."""
    NULL_MD5 = "null-md5"
    NULL_SHA1 = "null-sha1"
    NULL_SHA256 = "null-sha256"
    NULL_SHA384 = "null-sha384"
    NULL_SHA512 = "null-sha512"
    DES_NULL = "des-null"
    DES_MD5 = "des-md5"
    DES_SHA1 = "des-sha1"
    DES_SHA256 = "des-sha256"
    DES_SHA384 = "des-sha384"
    DES_SHA512 = "des-sha512"
    V_3DES_NULL = "3des-null"
    V_3DES_MD5 = "3des-md5"
    V_3DES_SHA1 = "3des-sha1"
    V_3DES_SHA256 = "3des-sha256"
    V_3DES_SHA384 = "3des-sha384"
    V_3DES_SHA512 = "3des-sha512"
    AES128_NULL = "aes128-null"
    AES128_MD5 = "aes128-md5"
    AES128_SHA1 = "aes128-sha1"
    AES128_SHA256 = "aes128-sha256"
    AES128_SHA384 = "aes128-sha384"
    AES128_SHA512 = "aes128-sha512"
    AES128GCM = "aes128gcm"
    AES192_NULL = "aes192-null"
    AES192_MD5 = "aes192-md5"
    AES192_SHA1 = "aes192-sha1"
    AES192_SHA256 = "aes192-sha256"
    AES192_SHA384 = "aes192-sha384"
    AES192_SHA512 = "aes192-sha512"
    AES256_NULL = "aes256-null"
    AES256_MD5 = "aes256-md5"
    AES256_SHA1 = "aes256-sha1"
    AES256_SHA256 = "aes256-sha256"
    AES256_SHA384 = "aes256-sha384"
    AES256_SHA512 = "aes256-sha512"
    AES256GCM = "aes256gcm"
    CHACHA20POLY1305 = "chacha20poly1305"
    ARIA128_NULL = "aria128-null"
    ARIA128_MD5 = "aria128-md5"
    ARIA128_SHA1 = "aria128-sha1"
    ARIA128_SHA256 = "aria128-sha256"
    ARIA128_SHA384 = "aria128-sha384"
    ARIA128_SHA512 = "aria128-sha512"
    ARIA192_NULL = "aria192-null"
    ARIA192_MD5 = "aria192-md5"
    ARIA192_SHA1 = "aria192-sha1"
    ARIA192_SHA256 = "aria192-sha256"
    ARIA192_SHA384 = "aria192-sha384"
    ARIA192_SHA512 = "aria192-sha512"
    ARIA256_NULL = "aria256-null"
    ARIA256_MD5 = "aria256-md5"
    ARIA256_SHA1 = "aria256-sha1"
    ARIA256_SHA256 = "aria256-sha256"
    ARIA256_SHA384 = "aria256-sha384"
    ARIA256_SHA512 = "aria256-sha512"
    SEED_NULL = "seed-null"
    SEED_MD5 = "seed-md5"
    SEED_SHA1 = "seed-sha1"
    SEED_SHA256 = "seed-sha256"
    SEED_SHA384 = "seed-sha384"
    SEED_SHA512 = "seed-sha512"

class Phase2InterfaceDhgrpEnum(str, Enum):
    """Allowed values for dhgrp field."""
    V_1 = "1"
    V_2 = "2"
    V_5 = "5"
    V_14 = "14"
    V_15 = "15"
    V_16 = "16"
    V_17 = "17"
    V_18 = "18"
    V_19 = "19"
    V_20 = "20"
    V_21 = "21"
    V_27 = "27"
    V_28 = "28"
    V_29 = "29"
    V_30 = "30"
    V_31 = "31"
    V_32 = "32"

class Phase2InterfaceAddke1Enum(str, Enum):
    """Allowed values for addke1 field."""
    V_0 = "0"
    V_35 = "35"
    V_36 = "36"
    V_37 = "37"
    V_1080 = "1080"
    V_1081 = "1081"
    V_1082 = "1082"
    V_1083 = "1083"
    V_1084 = "1084"
    V_1085 = "1085"
    V_1089 = "1089"
    V_1090 = "1090"
    V_1091 = "1091"
    V_1092 = "1092"
    V_1093 = "1093"
    V_1094 = "1094"

class Phase2InterfaceAddke2Enum(str, Enum):
    """Allowed values for addke2 field."""
    V_0 = "0"
    V_35 = "35"
    V_36 = "36"
    V_37 = "37"
    V_1080 = "1080"
    V_1081 = "1081"
    V_1082 = "1082"
    V_1083 = "1083"
    V_1084 = "1084"
    V_1085 = "1085"
    V_1089 = "1089"
    V_1090 = "1090"
    V_1091 = "1091"
    V_1092 = "1092"
    V_1093 = "1093"
    V_1094 = "1094"

class Phase2InterfaceAddke3Enum(str, Enum):
    """Allowed values for addke3 field."""
    V_0 = "0"
    V_35 = "35"
    V_36 = "36"
    V_37 = "37"
    V_1080 = "1080"
    V_1081 = "1081"
    V_1082 = "1082"
    V_1083 = "1083"
    V_1084 = "1084"
    V_1085 = "1085"
    V_1089 = "1089"
    V_1090 = "1090"
    V_1091 = "1091"
    V_1092 = "1092"
    V_1093 = "1093"
    V_1094 = "1094"

class Phase2InterfaceAddke4Enum(str, Enum):
    """Allowed values for addke4 field."""
    V_0 = "0"
    V_35 = "35"
    V_36 = "36"
    V_37 = "37"
    V_1080 = "1080"
    V_1081 = "1081"
    V_1082 = "1082"
    V_1083 = "1083"
    V_1084 = "1084"
    V_1085 = "1085"
    V_1089 = "1089"
    V_1090 = "1090"
    V_1091 = "1091"
    V_1092 = "1092"
    V_1093 = "1093"
    V_1094 = "1094"

class Phase2InterfaceAddke5Enum(str, Enum):
    """Allowed values for addke5 field."""
    V_0 = "0"
    V_35 = "35"
    V_36 = "36"
    V_37 = "37"
    V_1080 = "1080"
    V_1081 = "1081"
    V_1082 = "1082"
    V_1083 = "1083"
    V_1084 = "1084"
    V_1085 = "1085"
    V_1089 = "1089"
    V_1090 = "1090"
    V_1091 = "1091"
    V_1092 = "1092"
    V_1093 = "1093"
    V_1094 = "1094"

class Phase2InterfaceAddke6Enum(str, Enum):
    """Allowed values for addke6 field."""
    V_0 = "0"
    V_35 = "35"
    V_36 = "36"
    V_37 = "37"
    V_1080 = "1080"
    V_1081 = "1081"
    V_1082 = "1082"
    V_1083 = "1083"
    V_1084 = "1084"
    V_1085 = "1085"
    V_1089 = "1089"
    V_1090 = "1090"
    V_1091 = "1091"
    V_1092 = "1092"
    V_1093 = "1093"
    V_1094 = "1094"

class Phase2InterfaceAddke7Enum(str, Enum):
    """Allowed values for addke7 field."""
    V_0 = "0"
    V_35 = "35"
    V_36 = "36"
    V_37 = "37"
    V_1080 = "1080"
    V_1081 = "1081"
    V_1082 = "1082"
    V_1083 = "1083"
    V_1084 = "1084"
    V_1085 = "1085"
    V_1089 = "1089"
    V_1090 = "1090"
    V_1091 = "1091"
    V_1092 = "1092"
    V_1093 = "1093"
    V_1094 = "1094"

class Phase2InterfaceSrcAddrTypeEnum(str, Enum):
    """Allowed values for src_addr_type field."""
    SUBNET = "subnet"
    RANGE = "range"
    IP = "ip"
    NAME = "name"
    SUBNET6 = "subnet6"
    RANGE6 = "range6"
    IP6 = "ip6"
    NAME6 = "name6"

class Phase2InterfaceDstAddrTypeEnum(str, Enum):
    """Allowed values for dst_addr_type field."""
    SUBNET = "subnet"
    RANGE = "range"
    IP = "ip"
    NAME = "name"
    SUBNET6 = "subnet6"
    RANGE6 = "range6"
    IP6 = "ip6"
    NAME6 = "name6"


# ============================================================================
# Main Model
# ============================================================================

class Phase2InterfaceModel(BaseModel):
    """
    Pydantic model for vpn/ipsec/phase2_interface configuration.
    
    Configure VPN autokey tunnel.
    
    Validation Rules:        - name: max_length=35 pattern=        - phase1name: max_length=15 pattern=        - dhcp_ipsec: pattern=        - proposal: pattern=        - pfs: pattern=        - dhgrp: pattern=        - addke1: pattern=        - addke2: pattern=        - addke3: pattern=        - addke4: pattern=        - addke5: pattern=        - addke6: pattern=        - addke7: pattern=        - replay: pattern=        - keepalive: pattern=        - auto_negotiate: pattern=        - add_route: pattern=        - inbound_dscp_copy: pattern=        - auto_discovery_sender: pattern=        - auto_discovery_forwarder: pattern=        - keylifeseconds: min=120 max=172800 pattern=        - keylifekbs: min=5120 max=4294967295 pattern=        - keylife_type: pattern=        - single_source: pattern=        - route_overlap: pattern=        - encapsulation: pattern=        - l2tp: pattern=        - comments: max_length=255 pattern=        - initiator_ts_narrow: pattern=        - diffserv: pattern=        - diffservcode: pattern=        - protocol: min=0 max=255 pattern=        - src_name: max_length=79 pattern=        - src_name6: max_length=79 pattern=        - src_addr_type: pattern=        - src_start_ip: pattern=        - src_start_ip6: pattern=        - src_end_ip: pattern=        - src_end_ip6: pattern=        - src_subnet: pattern=        - src_subnet6: pattern=        - src_port: min=0 max=65535 pattern=        - dst_name: max_length=79 pattern=        - dst_name6: max_length=79 pattern=        - dst_addr_type: pattern=        - dst_start_ip: pattern=        - dst_start_ip6: pattern=        - dst_end_ip: pattern=        - dst_end_ip6: pattern=        - dst_subnet: pattern=        - dst_subnet6: pattern=        - dst_port: min=0 max=65535 pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    name: str | None = Field(max_length=35, default=None, description="IPsec tunnel name.")    
    phase1name: str = Field(max_length=15, description="Phase 1 determines the options required for phase 2.")  # datasource: ['vpn.ipsec.phase1-interface.name']    
    dhcp_ipsec: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable DHCP-IPsec.")    
    proposal: list[Phase2InterfaceProposalEnum] = Field(description="Phase2 proposal.")    
    pfs: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable PFS feature.")    
    dhgrp: list[Phase2InterfaceDhgrpEnum] = Field(default_factory=list, description="Phase2 DH group.")    
    addke1: list[Phase2InterfaceAddke1Enum] = Field(default_factory=list, description="phase2 ADDKE1 group.")    
    addke2: list[Phase2InterfaceAddke2Enum] = Field(default_factory=list, description="phase2 ADDKE2 group.")    
    addke3: list[Phase2InterfaceAddke3Enum] = Field(default_factory=list, description="phase2 ADDKE3 group.")    
    addke4: list[Phase2InterfaceAddke4Enum] = Field(default_factory=list, description="phase2 ADDKE4 group.")    
    addke5: list[Phase2InterfaceAddke5Enum] = Field(default_factory=list, description="phase2 ADDKE5 group.")    
    addke6: list[Phase2InterfaceAddke6Enum] = Field(default_factory=list, description="phase2 ADDKE6 group.")    
    addke7: list[Phase2InterfaceAddke7Enum] = Field(default_factory=list, description="phase2 ADDKE7 group.")    
    replay: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable replay detection.")    
    keepalive: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable keep alive.")    
    auto_negotiate: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable IPsec SA auto-negotiation.")    
    add_route: Literal["phase1", "enable", "disable"] | None = Field(default="phase1", description="Enable/disable automatic route addition.")    
    inbound_dscp_copy: Literal["phase1", "enable", "disable"] | None = Field(default="phase1", description="Enable/disable copying of the DSCP in the ESP header to the inner IP header.")    
    auto_discovery_sender: Literal["phase1", "enable", "disable"] | None = Field(default="phase1", description="Enable/disable sending short-cut messages.")    
    auto_discovery_forwarder: Literal["phase1", "enable", "disable"] | None = Field(default="phase1", description="Enable/disable forwarding short-cut messages.")    
    keylifeseconds: int | None = Field(ge=120, le=172800, default=43200, description="Phase2 key life in time in seconds (120 - 172800).")    
    keylifekbs: int | None = Field(ge=5120, le=4294967295, default=5120, description="Phase2 key life in number of kilobytes of traffic (5120 - 4294967295).")    
    keylife_type: Literal["seconds", "kbs", "both"] | None = Field(default="seconds", description="Keylife type.")    
    single_source: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable single source IP restriction.")    
    route_overlap: Literal["use-old", "use-new", "allow"] | None = Field(default="use-new", description="Action for overlapping routes.")    
    encapsulation: Literal["tunnel-mode", "transport-mode"] | None = Field(default="tunnel-mode", description="ESP encapsulation mode.")    
    l2tp: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable L2TP over IPsec.")    
    comments: str | None = Field(max_length=255, default=None, description="Comment.")    
    initiator_ts_narrow: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable traffic selector narrowing for IKEv2 initiator.")    
    diffserv: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable applying DSCP value to the IPsec tunnel outer IP header.")    
    diffservcode: str | None = Field(default=None, description="DSCP value to be applied to the IPsec tunnel outer IP header.")    
    protocol: int | None = Field(ge=0, le=255, default=0, description="Quick mode protocol selector (1 - 255 or 0 for all).")    
    src_name: str = Field(max_length=79, description="Local proxy ID name.")  # datasource: ['firewall.address.name', 'firewall.addrgrp.name']    
    src_name6: str = Field(max_length=79, description="Local proxy ID name.")  # datasource: ['firewall.address6.name', 'firewall.addrgrp6.name']    
    src_addr_type: Phase2InterfaceSrcAddrTypeEnum | None = Field(default=Phase2InterfaceSrcAddrTypeEnum.SUBNET, description="Local proxy ID type.")    
    src_start_ip: str | None = Field(default="0.0.0.0", description="Local proxy ID start.")    
    src_start_ip6: str | None = Field(default="::", description="Local proxy ID IPv6 start.")    
    src_end_ip: str | None = Field(default="0.0.0.0", description="Local proxy ID end.")    
    src_end_ip6: str | None = Field(default="::", description="Local proxy ID IPv6 end.")    
    src_subnet: Any = Field(default="0.0.0.0 0.0.0.0", description="Local proxy ID subnet.")    
    src_subnet6: str | None = Field(default="::/0", description="Local proxy ID IPv6 subnet.")    
    src_port: int | None = Field(ge=0, le=65535, default=0, description="Quick mode source port (1 - 65535 or 0 for all).")    
    dst_name: str = Field(max_length=79, description="Remote proxy ID name.")  # datasource: ['firewall.address.name', 'firewall.addrgrp.name']    
    dst_name6: str = Field(max_length=79, description="Remote proxy ID name.")  # datasource: ['firewall.address6.name', 'firewall.addrgrp6.name']    
    dst_addr_type: Phase2InterfaceDstAddrTypeEnum | None = Field(default=Phase2InterfaceDstAddrTypeEnum.SUBNET, description="Remote proxy ID type.")    
    dst_start_ip: str | None = Field(default="0.0.0.0", description="Remote proxy ID IPv4 start.")    
    dst_start_ip6: str | None = Field(default="::", description="Remote proxy ID IPv6 start.")    
    dst_end_ip: str | None = Field(default="0.0.0.0", description="Remote proxy ID IPv4 end.")    
    dst_end_ip6: str | None = Field(default="::", description="Remote proxy ID IPv6 end.")    
    dst_subnet: Any = Field(default="0.0.0.0 0.0.0.0", description="Remote proxy ID IPv4 subnet.")    
    dst_subnet6: str | None = Field(default="::/0", description="Remote proxy ID IPv6 subnet.")    
    dst_port: int | None = Field(ge=0, le=65535, default=0, description="Quick mode destination port (1 - 65535 or 0 for all).")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    @field_validator('phase1name')
    @classmethod
    def validate_phase1name(cls, v: Any) -> Any:
        """
        Validate phase1name field.
        
        Datasource: ['vpn.ipsec.phase1-interface.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('src_name')
    @classmethod
    def validate_src_name(cls, v: Any) -> Any:
        """
        Validate src_name field.
        
        Datasource: ['firewall.address.name', 'firewall.addrgrp.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('src_name6')
    @classmethod
    def validate_src_name6(cls, v: Any) -> Any:
        """
        Validate src_name6 field.
        
        Datasource: ['firewall.address6.name', 'firewall.addrgrp6.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('dst_name')
    @classmethod
    def validate_dst_name(cls, v: Any) -> Any:
        """
        Validate dst_name field.
        
        Datasource: ['firewall.address.name', 'firewall.addrgrp.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('dst_name6')
    @classmethod
    def validate_dst_name6(cls, v: Any) -> Any:
        """
        Validate dst_name6 field.
        
        Datasource: ['firewall.address6.name', 'firewall.addrgrp6.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "Phase2InterfaceModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)
    # ========================================================================
    # Datasource Validation Methods
    # ========================================================================    
    async def validate_phase1name_references(self, client: Any) -> list[str]:
        """
        Validate phase1name references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - vpn/ipsec/phase1-interface        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = Phase2InterfaceModel(
            ...     phase1name="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_phase1name_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.vpn.ipsec.phase2_interface.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "phase1name", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.vpn.ipsec.phase1_interface.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Phase1Name '{value}' not found in "
                "vpn/ipsec/phase1-interface"
            )        
        return errors    
    async def validate_src_name_references(self, client: Any) -> list[str]:
        """
        Validate src_name references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - firewall/address        - firewall/addrgrp        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = Phase2InterfaceModel(
            ...     src_name="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_src_name_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.vpn.ipsec.phase2_interface.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "src_name", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.firewall.address.exists(value):
            found = True
        elif await client.api.cmdb.firewall.addrgrp.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Src-Name '{value}' not found in "
                "firewall/address or firewall/addrgrp"
            )        
        return errors    
    async def validate_src_name6_references(self, client: Any) -> list[str]:
        """
        Validate src_name6 references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - firewall/address6        - firewall/addrgrp6        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = Phase2InterfaceModel(
            ...     src_name6="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_src_name6_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.vpn.ipsec.phase2_interface.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "src_name6", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.firewall.address6.exists(value):
            found = True
        elif await client.api.cmdb.firewall.addrgrp6.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Src-Name6 '{value}' not found in "
                "firewall/address6 or firewall/addrgrp6"
            )        
        return errors    
    async def validate_dst_name_references(self, client: Any) -> list[str]:
        """
        Validate dst_name references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - firewall/address        - firewall/addrgrp        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = Phase2InterfaceModel(
            ...     dst_name="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_dst_name_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.vpn.ipsec.phase2_interface.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "dst_name", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.firewall.address.exists(value):
            found = True
        elif await client.api.cmdb.firewall.addrgrp.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Dst-Name '{value}' not found in "
                "firewall/address or firewall/addrgrp"
            )        
        return errors    
    async def validate_dst_name6_references(self, client: Any) -> list[str]:
        """
        Validate dst_name6 references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - firewall/address6        - firewall/addrgrp6        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = Phase2InterfaceModel(
            ...     dst_name6="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_dst_name6_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.vpn.ipsec.phase2_interface.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "dst_name6", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.firewall.address6.exists(value):
            found = True
        elif await client.api.cmdb.firewall.addrgrp6.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Dst-Name6 '{value}' not found in "
                "firewall/address6 or firewall/addrgrp6"
            )        
        return errors    
    async def validate_all_references(self, client: Any) -> list[str]:
        """
        Validate ALL datasource references in this model.
        
        Convenience method that runs all validate_*_references() methods
        and aggregates the results.
        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of all validation errors found
            
        Example:
            >>> errors = await policy.validate_all_references(fgt._client)
            >>> if errors:
            ...     for error in errors:
            ...         print(f"  - {error}")
        """
        all_errors = []
        
        errors = await self.validate_phase1name_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_src_name_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_src_name6_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_dst_name_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_dst_name6_references(client)
        all_errors.extend(errors)        
        return all_errors

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "Phase2InterfaceModel",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:11.245197Z
# ============================================================================


"""
Pydantic Models for CMDB - user/group

Runtime validation models for user/group configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Literal, Optional
from enum import Enum

# ============================================================================
# Enum Definitions for Child Table Fields (for fields with 4+ allowed values)
# ============================================================================

# ============================================================================
# Child Table Models (sorted deepest-first so nested models are defined before their parents)
# ============================================================================

class GroupMember(BaseModel):
    """
    Child table model for member.
    
    Names of users, peers, LDAP severs, RADIUS servers or external idp servers to add to the user group.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str = Field(max_length=511, description="Group member name.")  # datasource: ['user.peer.name', 'user.local.name', 'user.radius.name', 'user.tacacs+.name', 'user.ldap.name', 'user.saml.name', 'user.external-identity-provider.name', 'user.adgrp.name', 'user.pop3.name', 'user.certificate.name']
class GroupMatch(BaseModel):
    """
    Child table model for match.
    
    Group matches.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="ID.")    
    server_name: str = Field(max_length=35, description="Name of remote auth server.")  # datasource: ['user.radius.name', 'user.ldap.name', 'user.tacacs+.name', 'user.saml.name', 'user.external-identity-provider.name']    
    group_name: str = Field(max_length=511, description="Name of matching user or group on remote authentication server or SCIM.")
class GroupGuest(BaseModel):
    """
    Child table model for guest.
    
    Guest User.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="Guest ID.")    
    user_id: str | None = Field(max_length=64, default=None, description="Guest ID.")    
    name: str | None = Field(max_length=64, default=None, description="Guest name.")    
    password: Any = Field(max_length=128, default=None, description="Guest password.")    
    mobile_phone: str | None = Field(max_length=35, default=None, description="Mobile phone.")    
    sponsor: str | None = Field(max_length=35, default=None, description="Set the action for the sponsor guest user field.")    
    company: str | None = Field(max_length=35, default=None, description="Set the action for the company guest user field.")    
    email: str | None = Field(max_length=64, default=None, description="Email.")    
    expiration: str | None = Field(default=None, description="Expire time.")    
    comment: str | None = Field(max_length=255, default=None, description="Comment.")
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================

class GroupGroupTypeEnum(str, Enum):
    """Allowed values for group_type field."""
    FIREWALL = "firewall"
    FSSO_SERVICE = "fsso-service"
    RSSO = "rsso"
    GUEST = "guest"


# ============================================================================
# Main Model
# ============================================================================

class GroupModel(BaseModel):
    """
    Pydantic model for user/group configuration.
    
    Configure user groups.
    
    Validation Rules:        - name: max_length=35 pattern=        - id_: min=0 max=4294967295 pattern=        - group_type: pattern=        - authtimeout: min=0 max=43200 pattern=        - auth_concurrent_override: pattern=        - auth_concurrent_value: min=0 max=100 pattern=        - http_digest_realm: max_length=35 pattern=        - sso_attribute_value: max_length=511 pattern=        - member: pattern=        - match: pattern=        - user_id: pattern=        - password: pattern=        - user_name: pattern=        - sponsor: pattern=        - company: pattern=        - email: pattern=        - mobile_phone: pattern=        - sms_server: pattern=        - sms_custom_server: max_length=35 pattern=        - expire_type: pattern=        - expire: min=1 max=31536000 pattern=        - max_accounts: min=0 max=500 pattern=        - multiple_guest_add: pattern=        - guest: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    name: str | None = Field(max_length=35, default=None, description="Group name.")    
    id_: int | None = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="Group ID.")    
    group_type: GroupGroupTypeEnum | None = Field(default=GroupGroupTypeEnum.FIREWALL, description="Set the group to be for firewall authentication, FSSO, RSSO, or guest users.")    
    authtimeout: int | None = Field(ge=0, le=43200, default=0, description="Authentication timeout in minutes for this user group. 0 to use the global user setting auth-timeout.")    
    auth_concurrent_override: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable overriding the global number of concurrent authentication sessions for this user group.")    
    auth_concurrent_value: int | None = Field(ge=0, le=100, default=0, description="Maximum number of concurrent authenticated connections per user (0 - 100).")    
    http_digest_realm: str | None = Field(max_length=35, default=None, description="Realm attribute for MD5-digest authentication.")    
    sso_attribute_value: str | None = Field(max_length=511, default=None, description="RADIUS attribute value.")    
    member: list[GroupMember] = Field(default_factory=list, description="Names of users, peers, LDAP severs, RADIUS servers or external idp servers to add to the user group.")    
    match: list[GroupMatch] = Field(default_factory=list, description="Group matches.")    
    user_id: Literal["email", "auto-generate", "specify"] | None = Field(default="email", description="Guest user ID type.")    
    password: Literal["auto-generate", "specify", "disable"] | None = Field(default="auto-generate", description="Guest user password type.")    
    user_name: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable the guest user name entry.")    
    sponsor: Literal["optional", "mandatory", "disabled"] | None = Field(default="optional", description="Set the action for the sponsor guest user field.")    
    company: Literal["optional", "mandatory", "disabled"] | None = Field(default="optional", description="Set the action for the company guest user field.")    
    email: Literal["disable", "enable"] | None = Field(default="enable", description="Enable/disable the guest user email address field.")    
    mobile_phone: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable the guest user mobile phone number field.")    
    sms_server: Literal["fortiguard", "custom"] | None = Field(default="fortiguard", description="Send SMS through FortiGuard or other external server.")    
    sms_custom_server: str | None = Field(max_length=35, default=None, description="SMS server.")  # datasource: ['system.sms-server.name']    
    expire_type: Literal["immediately", "first-successful-login"] | None = Field(default="immediately", description="Determine when the expiration countdown begins.")    
    expire: int | None = Field(ge=1, le=31536000, default=14400, description="Time in seconds before guest user accounts expire (1 - 31536000).")    
    max_accounts: int | None = Field(ge=0, le=500, default=0, description="Maximum number of guest accounts that can be created for this group (0 means unlimited).")    
    multiple_guest_add: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable addition of multiple guests.")    
    guest: list[GroupGuest] = Field(default_factory=list, description="Guest User.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    @field_validator('sms_custom_server')
    @classmethod
    def validate_sms_custom_server(cls, v: Any) -> Any:
        """
        Validate sms_custom_server field.
        
        Datasource: ['system.sms-server.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "GroupModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)
    # ========================================================================
    # Datasource Validation Methods
    # ========================================================================    
    async def validate_member_references(self, client: Any) -> list[str]:
        """
        Validate member references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - user/peer        - user/local        - user/radius        - user/tacacs+        - user/ldap        - user/saml        - user/external-identity-provider        - user/adgrp        - user/pop3        - user/certificate        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = GroupModel(
            ...     member=[{"name": "invalid-name"}],
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_member_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.user.group.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate child table items
        values = getattr(self, "member", [])
        if not values:
            return errors
        
        for item in values:
            if isinstance(item, dict):
                value = item.get("name")
            else:
                value = getattr(item, "name", None)
            
            if not value:
                continue
            
            # Check all datasource endpoints
            found = False
            if await client.api.cmdb.user.peer.exists(value):
                found = True
            elif await client.api.cmdb.user.local.exists(value):
                found = True
            elif await client.api.cmdb.user.radius.exists(value):
                found = True
            elif await client.api.cmdb.user.tacacs_plus.exists(value):
                found = True
            elif await client.api.cmdb.user.ldap.exists(value):
                found = True
            elif await client.api.cmdb.user.saml.exists(value):
                found = True
            elif await client.api.cmdb.user.external_identity_provider.exists(value):
                found = True
            elif await client.api.cmdb.user.adgrp.exists(value):
                found = True
            elif await client.api.cmdb.user.pop3.exists(value):
                found = True
            elif await client.api.cmdb.user.certificate.exists(value):
                found = True
            
            if not found:
                errors.append(
                    f"Member '{value}' not found in "
                    "user/peer or user/local or user/radius or user/tacacs+ or user/ldap or user/saml or user/external-identity-provider or user/adgrp or user/pop3 or user/certificate"
                )        
        return errors    
    async def validate_match_references(self, client: Any) -> list[str]:
        """
        Validate match references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - user/radius        - user/ldap        - user/tacacs+        - user/saml        - user/external-identity-provider        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = GroupModel(
            ...     match=[{"server-name": "invalid-name"}],
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_match_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.user.group.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate child table items
        values = getattr(self, "match", [])
        if not values:
            return errors
        
        for item in values:
            if isinstance(item, dict):
                value = item.get("server-name")
            else:
                value = getattr(item, "server-name", None)
            
            if not value:
                continue
            
            # Check all datasource endpoints
            found = False
            if await client.api.cmdb.user.radius.exists(value):
                found = True
            elif await client.api.cmdb.user.ldap.exists(value):
                found = True
            elif await client.api.cmdb.user.tacacs_plus.exists(value):
                found = True
            elif await client.api.cmdb.user.saml.exists(value):
                found = True
            elif await client.api.cmdb.user.external_identity_provider.exists(value):
                found = True
            
            if not found:
                errors.append(
                    f"Match '{value}' not found in "
                    "user/radius or user/ldap or user/tacacs+ or user/saml or user/external-identity-provider"
                )        
        return errors    
    async def validate_sms_custom_server_references(self, client: Any) -> list[str]:
        """
        Validate sms_custom_server references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/sms-server        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = GroupModel(
            ...     sms_custom_server="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_sms_custom_server_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.user.group.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "sms_custom_server", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.system.sms_server.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Sms-Custom-Server '{value}' not found in "
                "system/sms-server"
            )        
        return errors    
    async def validate_all_references(self, client: Any) -> list[str]:
        """
        Validate ALL datasource references in this model.
        
        Convenience method that runs all validate_*_references() methods
        and aggregates the results.
        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of all validation errors found
            
        Example:
            >>> errors = await policy.validate_all_references(fgt._client)
            >>> if errors:
            ...     for error in errors:
            ...         print(f"  - {error}")
        """
        all_errors = []
        
        errors = await self.validate_member_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_match_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_sms_custom_server_references(client)
        all_errors.extend(errors)        
        return all_errors

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "GroupModel",    "GroupMember",    "GroupMatch",    "GroupGuest",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:13.640280Z
# ============================================================================